PROGRAM MAIN
! 
! program MAIN is the main program in
! an implementation of a sigma-coordinate model
! on the Osteroy-Byfjorden system close to Bergen, NORWAY
! 
! build a fresh bom with the commands  make clean, make mod , make.
! this must typically be done when you make changes in the modules
! or move the code to another platform. when only minor changes are
! done to the code, a single "make" will do.  
!
! 
! Variables are declared in mod_main.f90 that contains the
! application specific variables and  which uses the variables
! defined in mod_bound.f90 and state.f90
! 
  USE MOD_MAIN
  IMPLICIT NONE
!
! open all datafiles
!
  CALL FILES
!
!  Read variable and parameter settings from datafile
!
  CALL SETUP
!
!  DYPGRIN computes H, HU, HV, FSM, DUM, DVM, COR, CBC.
!  DEPTH is called from DYGRIN to compute Z,ZZ,DZ,DZR and DZZ.
!  INto DYPGRIN goes DEPMIN and NFILTER and DSBOT.
!  Out comes DEPMAX
!
  CALL DYPGRIN(DEPMIN,NFILTER,DEPMAX,DSBOT)
!
! Count the number of ocean cells, and store their locations in 2D arrays.
!
  CALL condition 
!
!  Dump H to bottom.plt for plotting in TECPLOT
!
  CALL OUTBOTTOMFEM
!
! After DEPMAX is computed from H, the Courant number may be
! computed.
!
  COURANT = (DT/N2D)*SQRT(2.*GRAV*DEPMAX)/DX
  WRITE(*,*)'The 2-D Courant number is',COURANT
  IF (COURANT.GT.1.0) THEN
    WRITE(*,*)'The 2-D Courant number is too large'
    STOP
  ENDIF
!
!   If TIDE , then initialize tidal fields.
!
  IF(TIDE) THEN
     WRITE(*,*)'THE SIMULATION INCL. ',KCON,' TIDAL CONSTITUENT(S)'
     CALL TIDEINIT
  ENDIF
! 
! FIELDINIT is called to initialize all prognostic fields.
!
  CALL FIELDINIT
!
!  Call NCALFA to compute the relaxation parameters 
!  ALPHA and ALPHAE to be
!  used in the FRS-zones at the open boundaries for the 3-D variables and
!  2-D variables respectively.
!
  CALL NCALFA
!
!  Initialize output for Tecplot.
!
  WRITE(82,*)'TITLE = "2 D Fields " '
  WRITE(82,*)'VARIABLES = "X" "Y" "ETA" "U2" "V2" "RHO2" "U5" "V5" "RHO5"  '
  CALL NRG3D(EKIN,VOLUME,MASS,EETA)
  WRITE(*,*)ISTEP,EKIN,VOLUME,MASS
  WRITE(43,*)'TITLE = "ENERGIES " '
  WRITE(43,*)'VARIABLES = "TIME" "KIN-E" "VOLUME" "MASS" "POT-E" '
  WRITE(43,*)'ZONE T="ENERGY"', 'I=',NUMSTEP+1
  WRITE(43,1005)ISTEP*DT/3600.,EKIN,VOLUME,MASS,EETA
1005 FORMAT(1X,F8.4,2X,E12.5,2X,E10.4,2X,E11.5,2X,F8.4)
  WRITE(83,*)'TITLE = "POINT " '
  WRITE(83,*)'VARIABLES = "TIME" "ETA" "U" "V" '
  WRITE(83,*)'ZONE T="POINT"', 'I=',NUMSTEP+1
  WRITE(83,1006)ISTEP*DT/(3600.*24),ETA(10,20),U(10,20,1),V(10,20,1)
  WRITE(84,*)'TITLE = "POINT2 " '
  WRITE(84,*)'VARIABLES = "TIME" "ETA" "U" "V" '
  WRITE(84,*)'ZONE T="POINT2"', 'I=',NUMSTEP+1
  WRITE(84,1006)ISTEP*DT/(3600.*24),ETA(13,11),U(13,11,1),V(13,11,1)
  WRITE(85,*)'TITLE = "POINT3 " '
  WRITE(85,*)'VARIABLES = "TIME" "ETA" "U" "V" '
  WRITE(85,*)'ZONE T="POINT3"', 'I=',NUMSTEP+1
  WRITE(85,1006)ISTEP*DT/(3600.*24),ETA(23,19),U(23,19,1),V(23,19,1)  
1006 FORMAT(1X,F8.4,2X,E11.4,2X,E10.4,2X,E10.4)
! Print sections initially
!
WRITE(91,*)'TITLE = "Section 1 " '
WRITE(91,*)'VARIABLES = "X" "Z" "U" "V" "W" "SPEED" "RHO" '
!

!##############################################################
! START TIMESTEPPING
  
  DO ISTEP = 1,NUMSTEP
    CALL CPU_TIME(T1)
!
!  Compute the time in days.
!
     TIME = DT*FLOAT(ISTEP)/86400.
     WRITE (*,*) ISTEP, "---------------------", TIME
!
!***Atmospheric part
!
    IF (WIND) THEN
        CALL WINDFORCING
    ENDIF
    IF (ATMPRESS) THEN
        CALL ATMOSP
        ETATFRS = ETATFRS + ETABFRS
    ENDIF
!
!***River runoff part.
!
    IF (RIVLOG) THEN
      CALL RIVERS(NDATE)
    ENDIF
!
!   update ndate every hour.
!
     IF(MOD(ISTEP,NDIVIS).EQ.0)THEN
        CALL POSTID4(NDATE(1),NDATE(2),NDATE(3),NDATE(4),1)
        NDATE(5)=NDATE(5)+1
     ENDIF
!
!  Update the density field.
!
     IF (UNESCO) THEN
       CALL DENSUNESCO
     ELSE
       CALL DENS
     ENDIF
!
!  Compute vertical diffusivities and viscositities
!
     CALL MY2HALV
!
!  Advect Q2 and Q2L if ADVTURB is true
!
     IF (ADVTURB) THEN
       CALL UPSTREAMQ(Q2)
       CALL UPSTREAMQ(Q2L)
     ENDIF
!
     CALL BOUND(6,NDATE)
!
!  Store old values.
!
     ETAP = ETA
     UPREV = U
     VPREV = V
     SPREV = S
     TPREV = T
!
     DELETAE = 0.
     DELETAI = 0.
     DELUE = 0.
     DELVE = 0.
     DELSE = 0.
     DELTE = 0.
     DELUI = 0.
     DELVI = 0.
     DELSI = 0.
     DELTI = 0.
!
!  Take an explicit predictor step in ETA, U, V, S and T
!  and store coorrections DELETA, DELUE, DELVE, DELSE and DELTE.
!
!
!  Compute contributions from the internal pressure from old values of RHO
!                                !
     CALL INTERNAL
!
!  Compute contributions from the advective terms in the momentum equations
!
     CALL SUPERBEEUV
!
!  In MODESPLIT the U and V fields are split in 2-D external parts and 3-D internal parts
!  and propagated in time
!
     CALL MODESPLIT(ISTEP,NDATE,1)
!
!  Call Diagnose each NDIVISth time step to test for abnormal values.
!
!    IF (MOD(ISTEP,NDIVIS).EQ.0) THEN
       CALL DIAGNOSE(1)
!    ENDIF
     CALL BOUND(3,NDATE)
     CALL UPDATEDD
!
! Include advection and horizontal diffusivity of S and T 
!
     IF (FOURTH) THEN
       CALL ADVECT(SPREV,S,DELSE,SAVER,1)
       CALL ADVECT(TPREV,T,DELTE,TAVER,2)
     ELSE
       CALL SUPERBEEF(SPREV,S,DELSE,SAVER,1)
       CALL SUPERBEEF(TPREV,T,DELTE,TAVER,2)
     ENDIF
     CALL BOUND(4,NDATE)
!    WRITE(*,*)MAXLOC(S),MAXVAL(S),MAXVAL(DELSE)
!    WRITE(*,*)MAXLOC(T),MAXVAL(T),MAXVAL(DELTE)
!
!    IF THETA3D GT 0 THEN CORRECT APPROXIMATION AT NEW TIME STEP
!
     IF (THETA3D .GT. 0.0) THEN
!
!  Update the density field.
!
       IF (UNESCO) THEN
         CALL DENSUNESCO
       ELSE
         CALL DENS
       ENDIF
!  Take an implicit corrector step in U, V, S and T
!  and store coorrections DELUI, DELVI, DELSI and DELTI.
!
!
!  Compute contributions from the internal pressure from new values of RHO
!                                !
       CALL INTERNAL
!
!  Compute contributions from the advective terms in the momentum equations
!
       CALL SUPERBEEUV
!
!  In MODESPLIT the U and V fields are split in 2-D external parts and 3-D internal parts
!  and propagated in time
!
!  Start from the previous water-level again 
!
       ETA = ETAP
!
       CALL MODESPLIT(ISTEP,NDATE,2)
!
       U = UPREV + THETA3D*DELUI + (1-THETA3D)*DELUE
       V = VPREV + THETA3D*DELVI + (1-THETA3D)*DELVE
       ETA = ETAP + THETA3D*DELETAI + (1-THETA3D)*DELETAE
       W = THETA3D*WI + (1-THETA3D)*WE
!
!      WRITE(*,*)'max corr-UV',ISTEP,MAXVAL(DELUE-DELUI),MAXVAL(DELVE-DELVI)
!
!  Call Diagnose each NDIVISth time step to test for abnormal values.
!
       CALL DIAGNOSE(1)
       CALL BOUND(3,NDATE)
       CALL UPDATEDD
       IF (FOURTH) THEN
         CALL ADVECT(SPREV,S,DELSI,SAVER,1)
         CALL ADVECT(TPREV,T,DELTI,TAVER,2)
       ELSE
         CALL SUPERBEEF(SPREV,S,DELSI,SAVER,1)
         CALL SUPERBEEF(TPREV,T,DELTI,TAVER,2)
       ENDIF
!      WRITE(*,*)MAXLOC(S),MAXVAL(S),MAXVAL(DELSI)
!      WRITE(*,*)MAXLOC(T),MAXVAL(T),MAXVAL(DELTI)
       S = SPREV + THETA3D*DELSI + (1.0-THETA3D)*DELSE
       T = TPREV + THETA3D*DELTI + (1.0-THETA3D)*DELTE
!      WRITE(*,*)'max corr-ST',ISTEP,MAXVAL(DELSE-DELSI),MAXVAL(DELTE-DELTI)
     ENDIF
!
!  Call WREAL to get the z-coordinate vertical velocities
!
     CALL WREAL
!
     TSURF = 0.
     SSURF = 0.
!
!  Vertical diffusion of S and T
!
     CALL VERTDIFF(S,SSURF)
     CALL VERTDIFF(T,TSURF)
     CALL BOUND(4,NDATE)
!
!  test on output
!
     CALL OUTPUT
     WRITE(*,*)'SMAX',MAXLOC(S),MAXVAL(S)
     WRITE(*,*)'SMIN',MINLOC(S),MINVAL(S)
     WRITE(*,*)'TMAX',MAXLOC(T),MAXVAL(T)
     WRITE(*,*)'TMIN',MINLOC(T),MINVAL(T)
!
!  END MAIN DO LOOP IN TIME
!
     CALL CPU_TIME(T2)
     TOTTIME=(TOTTIME+T2)-T1
     PRINT *,'This timestep:',t2-t1,' seconds'
  END DO

  PRINT*, 'CPU TIME (SECONDS) SINCE BEGINNING = ', TOTTIME

END PROGRAM MAIN

